#!/bin/ksh93
#########################################################################################################
# iDoctor VIOS Investigator
# Licensed Materials - Property of IBM
# (C) Copyright IBM Corp. 2000, 2012 All Rights Reserved.
#########################################################################################################
# This script will collect NPIV/FCstat data for all devices
# Parm 1: vfc list flag:
#				-1: Create a vfc list once and use it for all snapshots.
#				-A: Create a vfc list for each snapshot (default).
#				-F filename: Read the vfc list from this specified file.  Use this list for all snapshots.
# Parm 2: Number of snapshots/intervals (default is 6).
# Parm 3: Duration of each snapshot/interval (default is 600).
# Parm 4: cd will be issued to this location
# Parm 5: Description (optional)
# Parm 6: Filename
#
# MAIN
cmdstring=$(printf "%s %s %s %s %s %s %s %s %s %s %s" $0 $@ | awk '{print $0}')    #save off the entire command string
BASEDIR=$(dirname $0)	# This will get the directory this script resides in
ts=$(/usr/bin/date -u +"%Y-%m-%d-%H.%M.%S.000000") # Get a time stamp suitable for file naming.
vfcListFile="vfcList."$ts".txt" # Generate the default vfcList file name.
vfcListFlag=$1 # Get the vfc list flag.
if [[ $vfcListFlag == "-1" ]] then # If a single vfcList is to be generated,
	print "Collection will be based on the VFC list obtained at the start of the collection." # Inform user of choice.
	shift # Shift the parameters to the next parameter.
elif [[ $vfcListFlag == "-A" ]] then # If a vfcList is to be generated for each snapshot,
	print "Collections will be based on a VFC list obtained for each snapshot." # Inform user of choice.
	shift # Shift the parameters to the next parameter.
elif [[ $vfcListFlag == "-F" && -f $2 ]] then # If a vfcList to be obtained from a file, and file exists
	vfcListFile=$2 # Get the vfcList file name.
	print "Collection will be based on the VFC list obtained from $vfcListFile." # Inform user of choice.
	shift 2 # Shift the parameter to the next parameter.
elif [[ $vfcListFlag == "-F" ]] then # If a vfcList to be obtained from a file, and file DOES NOT exist
	vfcListFile=$2 # Get the vfcList file name.
	print "Specified file $vfcListFile does not exist. It will be created." # Inform user of choice.
	vfcListFlag="-F1"	# sets a special flag to indicate to not delete the file created
	shift 2 # Shift the parameter to the next parameter.
else # The vfcList flag was not specified.
	vfcListFlag="-A" # Set the default vfcList flag.
	print "Collections will be based on a VFC list obtained for each snapshot." # Inform user of choice.
fi
nSnaps=$1 # Get the number of snap shots.
duration=$2 # Get the duration of each snap shot.
description=$4 # get the description to save with this data
fileName=$5 # Get the file name where to place the snap shots.

$3  # change to the directory desired

if [[ $nSnaps == "" ]] then # If the number of snap shots was not specified, 
	nSnaps=6 # Set the number of snap shots to 6.
fi # End of If
if [[ $duration == "" ]] then # If the duration of each snap shot was not specified, 
	duration=600 # Set the duration to 10 minutes.
fi # End of If
if [[ $description == "" ]] then # If description was not supplied
	description="." # must be non blank or will mess 1st line output data formatting if future fields are added at end
fi # End of If
if [[ $fileName == "" ]] then # If the file name for containing the snap shots was not specified, 
	fileName="fc_npiv."$ts".npiv" # Set the file name to a unique name.
fi # End of If

printf "" >> $fileName

# write out general information about this collection
host_name=$(/usr/bin/hostname -s)
node_name=$(uname -n)
user_name=$(logname)
build=$(uname -s)
os_level=$(oslevel)
ser_num=`lsattr -El sys0 -a systemid | awk '{print $2}' | awk -F, '{print substr($2, 3)}'`
version=$(cat $BASEDIR/qidrvrm.txt | awk '{print $1}')   # retrieves build number (before the timestamp only)
description=$(expr substr "$description" 1 50)  # make sure description is just 50 bytes or less
node_name=$(expr substr "$node_name" 1 50)
user_name=$(expr substr "$user_name" 1 20)
build=$(expr substr "$build" 1 10)
version=$(expr substr "$version" 1 10)
cmdstring=$(expr substr "$cmdstring" 1 255)
printf "%255s%26s%10s%10s%255s%50s%50s%20s%10s%10s%10s%10s%255s" $host_name $ts $nSnaps $duration $vfcListFile "$description" $node_name $user_name $build $os_level $ser_num $version "$cmdstring" >> $fileName

print "Collecting a snapshot every $duration seconds for $nSnaps durations and saving the snapshots in $fileName." # Report the collection parameters.
startTime=$SECONDS # Get the current seconds.
if [[ $vfcListFlag != "-F" ]] then # If we need to generate a vfc list.
	$BASEDIR/getVfcList.sh $vfcListFile # Get a list of the FC devices with associated client partition names and wwpns.
fi # End of If.
if [[ $vfcListFlag != "-A" ]] then # If we are not generating a vfc list every snapshot,
	startTime=$SECONDS # Get the current seconds without the list generation.
fi # End of If.
interval=1 # Set the interval number to 1.

print "\ninterval,duration,timestamp,fcdev,vfcdev,hostname,wwpn,inreqs,outreqs,ctrlreqs,inbytes,outbytes,DMA_errs,Elem_errs,Comm_errs,IP inreqs,IP outreqs,IP ctrlreqs,IP inbytes,IP outbytes,IP DMA_errs,IP Elem_errs,Port speed (supported),Port Speed (supported) units,Port speed (running), Port Speed (running) units" >> $fileName # Place the headers in the file.
$BASEDIR/fc_snap.sh $vfcListFile $interval 0 >> $fileName # Collect the first snap shot.
let "delay = duration - SECONDS + startTime" # Calculate the delay based on how long it took to take the snapshot.
while [[ $interval -lt $nSnaps ]]; do # While there are intervals to collect,
	if [[ $delay < 0 ]] then 
		delay=0
	fi
	sleep $delay # Wait for the duration to pass. 
	let "interval = interval + 1" # Increment the interval number.
	startTime=$SECONDS # Get the current seconds.
	if [[ $vfcListFlag == "-A" ]] then # If generating a VFC list for each snapshot,
		$BASEDIR/getVfcList.sh $vfcListFile # Get a list of the FC devices with associated client partition names and wwpns.
	fi # End of If.
	$BASEDIR/fc_snap.sh $vfcListFile $interval $duration >> $fileName # Collect the snap shot.
	let "delay = duration - SECONDS + startTime" # Calculate the delay based on how long it took to take the snapshot.
done # End of While.
if [[ $vfcListFlag != "-F" && $vfcListFlag != "-F1" ]] then # If we generated the vfc list file,
	/usr/bin/rm $vfcListFile # Remove the vfc list file.
fi # End of If.